/**         @file  assert.h
 *         @brief  Debugging macros
 *          @date  11/05/1997
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 1997-2024, François Cerbelle
 *
 *  Originally inspired by "L'art du code", Steve Maguire, Microsoft Press
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-07-27 20:44
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __ASSERT_H__
#define __ASSERT_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef NDEBUG
#include <stdlib.h> /* abort */

#ifdef __cplusplus
extern "C" {
#endif

/** Print a debug trace (checkpoint)
 *
 *  @param[in] p_File Source file
 *  @param[in] p_Line Source line in the source file
 *  @param[in] p_CompilDate Compilation date
 *  @param[in] p_CompilTime Compilation time
 *  @param[in] p_Function Function name in the source file
 *
 *  Outputs on stderr a timestamp, with the filename, the sourceline,
 *  the compilation date and time, the function name.
 */
void _trace(const char *p_File,
            const unsigned int p_Line,
            const char *p_CompilDate,
            const char *p_CompilTime, const char *p_Function);

/** Print a debug trace (checkpoint) with a static message
 *
 *  @param[in] p_File Source file
 *  @param[in] p_Line Source line in the source file
 *  @param[in] p_CompilDate Compilation date
 *  @param[in] p_CompilTime Compilation time
 *  @param[in] p_Function Function name in the source file
 *  @param[in] p_Message Static message
 *
 *  Outputs on stderr a timestamp, with the filename, the sourceline,
 *  the compilation date and time, the function name and a static
 *  message.
 */
void _trace_msg(const char *p_File,
                const unsigned int p_Line,
                const char *p_CompilDate,
                const char *p_CompilTime,
                const char *p_Function, const char *p_Message);

/** Print a debug trace (checkpoint) with a formatted  message
 *
 *  @param[in] p_File Source file
 *  @param[in] p_Line Source line in the source file
 *  @param[in] p_CompilDate Compilation date
 *  @param[in] p_CompilTime Compilation time
 *  @param[in] p_Function Function name in the source file
 *  @param[in] p_Format format string
 *  @param[in] ... Formatted string parameters
 *
 *  Outputs on stderr a timestamp, with the filename, the sourceline,
 *  the compilation date and time, the function name and a formatted
 *  message.
 */
void _trace_dynmsg(const char *p_File,
                   const unsigned int p_Line,
                   const char *p_CompilDate,
                   const char *p_CompilTime,
                   const char *p_Function, const char *p_Format, ...
                  );

#ifdef __cplusplus
}
#endif

#    endif                      /* NDEBUG */

/**  Assertion check macro
 *
 *  @param[in] condition to check
 *
 * If NDEBUG is set, does nothing. If NDEBUG is not defined, checks that the
 * condition is true, otherwise stop the process
 */
#ifndef NDEBUG
#define ASSERT(condition) \
    if (!(condition)) { \
        _trace_dynmsg(__FILE__,__LINE__, __DATE__, __TIME__, __func__,"Assertion failed (%s)" , #condition); \
        abort(); \
    }
#else                         /* NDEBUG */
#define ASSERT(condition)
#endif                        /* NDEBUG */

/**  Checkpoint on stderr
 *
 *  Writes a timestamped checkpoint with filename, function and line
 *  number on stderr.
 */
#ifndef NDEBUG
#define DBG_TRACE \
    _trace(__FILE__,__LINE__, __DATE__, __TIME__, __func__)
#else                         /* NDEBUG */
#define DBG_TRACE
#endif                        /* NDEBUG */

/**  Checkpoint on stderr with a static message
 *
 *  Writes a timestamped checkpoint with filename, function and line
 *  number on stderr.
 */
#ifndef NDEBUG
#define DBG_MSG(msg) \
    _trace_msg(__FILE__,__LINE__, __DATE__, __TIME__, __func__,msg)
#else                         /* NDEBUG */
#define DBG_MSG(msg)
#endif                        /* NDEBUG */

/**  Instruction checkpint
 *
 *  Writes a checkpoint trace with timestamp, filename, function name
 *  and line number when executing an instruction.
 */
#ifndef NDEBUG
#define DBG_ITRACE(inst) \
  _trace_msg(__FILE__,__LINE__, __DATE__, __TIME__, __func__,#inst), \
  inst
#else                         /* NDEBUG */
#define DBG_ITRACE(inst) inst
#endif                        /* NDEBUG */

/**  Log a timestamped debugging message on stderr
 *
 *  Writes a timestamped message on stderr with the filename, function
 *  name, line number.
 */
#ifndef NDEBUG
#define DBG_PRINTF(p_Format, ...) \
  _trace_dynmsg(__FILE__,__LINE__, __DATE__, __TIME__, __func__, p_Format, __VA_ARGS__)
#else                         /* NDEBUG */
#define DBG_PRINTF(p_Format, ...)
#endif                        /* NDEBUG */

#endif                          /* _ASSERT_H */

