/**         @file  memory.h
 *         @brief  Tracks memory allocation and leaks when compiled without NDEBUG
 *          @date  25/09/2006
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 2017-2024, François Cerbelle
 *
 *  Originally inspired by "L'art du code", Steve Maguire, Microsoft Press
 *   This header needs to be included after most of the standard headers, ideally the last one.
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-07-28 15:44
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __MEMORY_H__
#define __MEMORY_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifndef NDEBUG

#include "memdbg.h"

/** Same syntaxt and same behavior than regular malloc function, with memory leaks tracking */
#define malloc(size) dbg_malloc(size,__FILE__,__LINE__,__DATE__,__TIME__,__func__)

/** Same syntaxt and same behavior than regular realloc function, with memory leaks tracking */
#define realloc(ptr,size) dbg_realloc(ptr,size,__FILE__,__LINE__,__DATE__,__TIME__,__func__)

/** Same syntaxt and same behavior than regular calloc function, with memory leaks tracking */
#define calloc(nmemb,size) dbg_calloc(nmemb,size,__FILE__,__LINE__,__DATE__,__TIME__,__func__)

/** Same syntaxt and same behavior than regular free function, with memory leaks tracking */
#define free(ptr) dbg_free(ptr,__FILE__,__LINE__,__DATE__,__TIME__,__func__)

/** Same syntaxt and same behavior than regular strdup function, with memory leaks tracking */
#define strdup(chaine) dbg_strdup(chaine,__FILE__,__LINE__,__DATE__,__TIME__,__func__)

/** Same syntaxt and same behavior than regular asprintf function, with memory leaks tracking */
#define asprintf(out,format,...) dbg_asprintf(out,format,__FILE__,__LINE__,__DATE__,__TIME__,__func__,__VA_ARGS__)

/** Prints a list of currently allocated blocks on stderr */
#define memreport() memtrack_dumpblocks()

#else

/** Empty NOOP replacement for memreport if compiled with NDEBUG */
#define memreport()

#endif                          /* NDEBUG */
#endif                          /* __MEMORY_H__ */

